local RaidAchFilter = LibStub("AceAddon-3.0"):NewAddon("RaidAchFilter", "AceConsole-3.0", "AceEvent-3.0")
local icon = LibStub("LibDBIcon-1.0")

-- DataBroker Entry
local LDB = LibStub("LibDataBroker-1.1"):NewDataObject("RaidAchievementFilter", {
	type = "launcher",
	label = "RaidAchievementFilter",
	icon = "Interface\\Icons\\Achievement_Dungeon_GloryoftheRaider",
	OnClick = function(self, button)
		if button == "RightButton" then
			InterfaceOptionsFrame_OpenToCategory("RaidAchieveFilter")
		else
			RaidAchFilter:Toggle()
		end
	end,
	OnTooltipShow = function(tt)
		tt:AddLine("RaidAchievementFilter")
		tt:AddLine("|cffffffff" .. "Click to Toggle, Right-Click for Options.")
	end,
})

-- Initialize
function RaidAchFilter:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RAchF_DB", {
		profile = {
			minimap = {
				hide = false,
			},
		},
	})
	icon:Register("RaidAchieveFilter", LDB, self.db.profile.minimap)
	RaidAchFilter:RegisterOptions()
end


-- Creating Blizzard Options
function RaidAchFilter:RegisterOptions()
	local AceConfig = LibStub("AceConfig-3.0")
	AceConfig:RegisterOptionsTable("RaidAchieveFilter", {
		type = 'group',
		args = {
			togglebutton = {
				type = 'toggle',
				order = 1,
				name = "Show minimap icon",
				desc = "Show/hide minimap icon.",
				get = function()
					return not RaidAchFilter.db.profile.minimap.hide
				end,
				set = RaidAchFilter.ToggleLDBIcon,
			},
		},
	})
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RaidAchieveFilter")
end

-- Toggle Minimap Icon
function RaidAchFilter:ToggleLDBIcon()
	RaidAchFilter.db.profile.minimap.hide = not RaidAchFilter.db.profile.minimap.hide
	if RaidAchFilter.db.profile.minimap.hide then
		icon:Hide("RaidAchieveFilter")
	else
		icon:Show("RaidAchieveFilter")
	end
end

--Slash Command
RaidAchFilter:RegisterChatCommand("rachf", "Toggle")

-- Toggle the Window
function RaidAchFilter:Toggle()
	local frame = _G["RaidAch_Frame"]
	local fra = _G["AchievementFrame"]
	if not frame then
		RaidAchFilter:Initialize(); RaidAchFilter:ShowAch(1); RaidAchFilter:ZoneChange()
		if fra then 
			if ( AchievementFrame:IsShown() ) then frame:ClearAllPoints(); frame:SetPoint("TOPLEFT",fra,"TOPRIGHT",4,0)end 
		end
	elseif frame:IsVisible() then 
		frame:Hide()
	else 
		frame:Show()
		if fra then 
			if ( AchievementFrame:IsShown() ) then frame:ClearAllPoints(); frame:SetPoint("TOPLEFT",fra,"TOPRIGHT",4,0) end 
		end
	end
	
end

--Event When Achievement Earned
function RaidAchFilter:AchieveEarned(eventName, achID)
	local fra = _G["RaidAch_Frame"]
	if fra then
		for i = 1, #RList do
			if RList[i] == achID then RaidAchFilter:ShowAch(RAF_RID) end
		end
	end
end
RaidAchFilter:RegisterEvent("ACHIEVEMENT_EARNED", "AchieveEarned")

--Auto swap to instance you're in
function RaidAchFilter:ZoneChange()
	local fra = _G["RaidAch_Frame"]
	if fra then
		local mapID = GetCurrentMapAreaID()
		for i = 1, #RAF_MapIDs do
			if RAF_MapIDs[i] == mapID then 
				local id = i
				if id < 5 or id == 7 or id == 8 or (id > 9 and id < 14) then
					local dif = GetRaidDifficultyID()
					if dif == 2 or diff == 4 then id = id+1; RaidAchFilter:ShowAch(id); break --25 Player
					else RaidAchFilter:ShowAch(id); break end --10 Player
				else
					RaidAchFilter:ShowAch(id); break
				end
				
			end
		end
	end
end
RaidAchFilter:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChange")

-- Creating the Window
function RaidAchFilter:Initialize()

	local frame = CreateFrame("Frame", "RaidAch_Frame", UIParent)
	frame:SetFrameStrata("DIALOG")
	frame:SetMovable(true)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", frame.StartMoving)
	frame:SetScript("OnDragStop", frame.StopMovingOrSizing)
	frame:SetPoint("CENTER"); 
	frame:SetWidth(520); frame:SetHeight(588);
	local tex = frame:CreateTexture("ARTWORK");
	tex:SetAllPoints();
	tex:SetTexture(0, 0, 0); tex:SetAlpha(0.8);
	frame:Show();

	local scrollframe = CreateFrame("ScrollFrame", "RaidAch_ScrollFrame", frame) 
	scrollframe:SetPoint("TOPLEFT", 10, -55) 
	scrollframe:SetPoint("BOTTOMRIGHT", -5, 10) 
	frame.scrollframe = scrollframe

	local scrollbar = CreateFrame("Slider", "RaidAch_Slider", scrollframe, "UIPanelScrollBarTemplate") 
	scrollbar:SetPoint("TOPLEFT", frame, "TOPRIGHT", 0, -16) 
	scrollbar:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", 0, 16) 
	scrollbar:SetMinMaxValues(1, 100) 
	scrollbar:SetValueStep(50) 
	scrollbar.scrollStep = 50
	scrollbar:SetValue(0) 
	scrollbar:SetWidth(16) 
	scrollbar:SetScript("OnValueChanged", 
		function (self, value) self:GetParent():SetVerticalScroll(value) end) 
	local scrollbg = scrollbar:CreateTexture(nil, "BACKGROUND") 
	scrollbg:SetAllPoints(scrollbar) 
	scrollbg:SetTexture(0, 0, 0, 0.8) 
	frame.scrollbar = scrollbar 
	scrollframe:EnableMouseWheel(true)
	scrollframe:SetScript("OnMouseWheel",
		function(self, delta) 
			local x = scrollbar:GetValue(); local y = scrollbar:GetValueStep()*3
			if #RList > 7 then -- Only work if scrollbar is up
				if delta == 1 then scrollbar:SetValue(x-y)
				else scrollbar:SetValue(x+y) end 
			end
		end)
	local content = CreateFrame("Frame", "RaidAch_Content", scrollframe) 
	content:SetSize(128, 128) 
	scrollframe.content = content 
	scrollframe:SetScrollChild(content)
	-- Border
	local backdrop = {
	  edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	  tile = false, tileSize = 8, edgeSize = 8,
	  insets = { left = 2, right = 2, top = 2, bottom = 2 }
	}
	frame:SetBackdrop(backdrop)
	-- Header Text
	local ttl = frame:CreateFontString("$parent_HeaderText","OVERLAY","GameFontNormal")
	ttl:SetPoint("TOPLEFT",frame,"TOPLEFT",8,-5);
	ttl:SetText("Raid Achievement Filter");
	ttl:Show();
	-- Close Button
	local btn = CreateFrame("BUTTON", nil, frame, "UIPanelCloseButton")
	btn:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 5, 5);
	btn:SetAlpha(1);
	btn:Show();
	btn:Enable();
	-- Hide Completed button
	local btn2 = CreateFrame("CheckButton", "$parent_Option", frame, "ChatConfigCheckButtonTemplate")
	btn2:SetPoint("TOPRIGHT", -105, -25);
	_G[btn2:GetName() .. "Text"]:SetText("Hide Completed")
	btn2:RegisterForClicks("AnyUp");
	btn2:SetScript("OnClick", function (self, button, down) 
	  if RAF_RID ~= nil then RaidAchFilter:ShowAch(RAF_RID); end
	end)
	-- Raids list dropdown
	local dropDown = CreateFrame("Button", "RaidAch_Dropdown", frame, "UIDropDownMenuTemplate")
	UIDropDownMenu_Initialize(dropDown)
	UIDropDownMenu_SetText(dropDown, "Filter by Instance")
	UIDropDownMenu_JustifyText(dropDown, "LEFT") 
	dropDown:SetPoint("TOPLEFT", frame, "TOPLEFT", -10, -20);
	UIDropDownMenu_SetWidth(dropDown, 220)	dropDown:SetAlpha(1);
	dropDown:SetScript("OnClick", function() 
		ToggleDropDownMenu(1, nil, dropDown, dropDown, 16, 5)
	end)

	dropDown.HideMenu = function()
	    if UIDROPDOWNMENU_OPEN_MENU == dropDown then
		CloseDropDownMenus()
	    end
	end

	local info = {}
	dropDown.initialize = function(self, level)
	    if not level then return end
		wipe(info)
		if level == 1 then
			info.isTitle = 1
			info.notCheckable = 1
			info.keepShownOnClick = false
			info.text = "Filter by Instance"
			UIDropDownMenu_AddButton(info, level)

			info.disabled = nil
			info.isTitle = nil
			info.notCheckable = true
			info.hasArrow = true

			info.text = "Raids"
			info.value = "topmenu1"
			UIDropDownMenu_AddButton(info, level)
			info.text = "Dungeons"
			info.value = "topmenu2"
			UIDropDownMenu_AddButton(info, level)
			info.text = "Scenarios"
			info.value = "topmenu3"
			UIDropDownMenu_AddButton(info, level)

			-- Close menu item
			info.hasArrow     = nil
			info.value        = nil
			info.notCheckable = 1
			info.text         = CLOSE
			info.func         = self.HideMenu
			UIDropDownMenu_AddButton(info, level)

		elseif level == 2 then
		
			info.disabled = nil
			info.isTitle = nil
			info.notCheckable = true
			info.hasArrow = true

			if UIDROPDOWNMENU_MENU_VALUE == "topmenu1" then
				info.text = "Lich King"
				info.value = "submenua1"
				UIDropDownMenu_AddButton(info, level)
				info.text = "Cataclysm"
				info.value = "submenua2"
				UIDropDownMenu_AddButton(info, level)
				info.text = "Mists of Pandaria"
				info.value = "submenua3"
				UIDropDownMenu_AddButton(info, level)
			elseif UIDROPDOWNMENU_MENU_VALUE == "topmenu2" then
				info.text = "Lich King"
				info.value = "submenub1"
				UIDropDownMenu_AddButton(info, level)
				info.text = "Cataclysm"
				info.value = "submenub2"
				UIDropDownMenu_AddButton(info, level)
				info.text = "Mists of Pandaria"
				info.value = "submenub3"
				UIDropDownMenu_AddButton(info, level)
			elseif UIDROPDOWNMENU_MENU_VALUE == "topmenu3" then
				for i = 62, 76 do --Scenarios!
				  info.value = i
				  info.checked = false
				  info.hasArrow = false
				  info.notCheckable = nil
				  info.text = RAF_Names[i]
				  info.func = function() 
					RaidAchFilter:ShowAch(i); ToggleDropDownMenu(1, nil, dropDown);
				  end
				  UIDropDownMenu_AddButton(info, level)
				end
			end
		 elseif level == 3 then
			-- Raids!
			if UIDROPDOWNMENU_MENU_VALUE == "submenua1" then
				for i = 1, 13 do
				  info.value = i
				  info.checked = false
				  info.text = RAF_Names[i]
				  info.func = function() 
					RaidAchFilter:ShowAch(i); ToggleDropDownMenu(1, nil, dropDown);
				  end
				  UIDropDownMenu_AddButton(info, level)
				end
			elseif UIDROPDOWNMENU_MENU_VALUE == "submenua2" then
				for i = 14, 18 do
				  info.value = i
				  info.checked = false
				  info.text = RAF_Names[i]
				  info.func = function() 
					RaidAchFilter:ShowAch(i); ToggleDropDownMenu(1, nil, dropDown);
				end
				  UIDropDownMenu_AddButton(info, level)
				end
			elseif UIDROPDOWNMENU_MENU_VALUE == "submenua3" then
				for i = 19, 22 do
				  info.value = i
				  info.checked = false
				  info.text = RAF_Names[i]
				  info.func = function() 
					RaidAchFilter:ShowAch(i); ToggleDropDownMenu(1, nil, dropDown);
				  end
				  UIDropDownMenu_AddButton(info, level)
				end
			-- Dungeons!
			elseif UIDROPDOWNMENU_MENU_VALUE == "submenub1" then
				for i = 23, 38 do
					  info.value = i
					  info.checked = false
					  info.text = RAF_Names[i]
					  info.func = function() 
					RaidAchFilter:ShowAch(i); ToggleDropDownMenu(1, nil, dropDown);
					  end
					  UIDropDownMenu_AddButton(info, level)
				end
			elseif UIDROPDOWNMENU_MENU_VALUE == "submenub2" then
				for i = 39, 52 do
					  info.value = i
					  info.checked = false
					  info.text = RAF_Names[i]
					  info.func = function() 
					RaidAchFilter:ShowAch(i); ToggleDropDownMenu(1, nil, dropDown);
					  end
					  UIDropDownMenu_AddButton(info, level)
				end
			elseif UIDROPDOWNMENU_MENU_VALUE == "submenub3" then
				for i = 53, 61 do
					  info.value = i
					  info.checked = false
					  info.text = RAF_Names[i]
					  info.func = function() 
					RaidAchFilter:ShowAch(i); ToggleDropDownMenu(1, nil, dropDown);
					  end
					  UIDropDownMenu_AddButton(info, level)
				end
			end
	     end
	 end
	--Set default when opened the first time
	UIDropDownMenu_Initialize(dropDown)
	UIDropDownMenu_SetSelectedValue(dropDown, 1)
	UIDropDownMenu_SetText(dropDown, "Filter by Instance")

end

function RaidAchFilter:AchAdd(fr_name, ach_ID, indent_num)

	local IDNumber, Name, Points, Completed, Month, Day, Year, Description, Flags, Image, RewardText, isGuildAch, earnedByMe, earnedBy = GetAchievementInfo(ach_ID)
	local frame = _G["RaidAch_Content"]
	
	local fra = _G[fr_name];
	if not fra then fra = CreateFrame("Frame", fr_name, frame); fra.texture = fra:CreateTexture(); end
	fra:SetSize(500,70)
	fra:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, indent_num)
	fra.texture:SetAllPoints(fra)
	if Completed == true then 
		fra.texture:SetTexture("Interface\\AchievementFrame\\UI-Achievement-AchievementBackground", "BACKGROUND")
		fra.texture:SetTexCoord(0, 1, 0, 0.5)
		fra:SetScript("OnMouseDown", function(self, button) 
			if button == "LeftButton" then 
				if ( IsModifiedClick("CHATLINK") ) then 
					if ChatEdit_GetActiveWindow() then ChatEdit_InsertLink(GetAchievementLink(ach_ID)) end
				else  
					if _G["AchievementFrame"] then
						if ( not AchievementFrame:IsShown() ) then
							AchievementFrame_ToggleAchievementFrame()
							AchievementFrame_SelectAchievement(ach_ID)
						else
							AchievementFrame_SelectAchievement(ach_ID)
						end
					end
				end
			end
		end)
	elseif Completed == false then
		fra.texture:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Parchment-Horizontal-Desaturated", "BACKGROUND")
		fra.texture:SetTexCoord(0, 1, 0, 0.5)
		fra:SetScript("OnMouseDown", function(self, button) 
		  if button == "LeftButton" then 
			if ( IsModifiedClick("CHATLINK") and ChatEdit_GetActiveWindow() ) then
				    ChatEdit_InsertLink(GetAchievementLink(ach_ID))
			elseif ( IsModifiedClick("QUESTWATCHTOGGLE") ) then
				    AddTrackedAchievement(ach_ID)
			else  
				if _G["AchievementFrame"] then
					if ( not AchievementFrame:IsShown() ) then
						AchievementFrame_ToggleAchievementFrame()
						AchievementFrame_SelectAchievement(ach_ID)
					else
						AchievementFrame_SelectAchievement(ach_ID)
					end
				end
			end
		  end
		end)
	end
	fra:SetScript("OnDragStart", frame.StartMoving)
	fra:SetScript("OnDragStop", frame.StopMovingOrSizing)
	fra:Show()

	local fraicon = _G[fr_name .. "icon"]
	if not fraicon then 
		fraicon = CreateFrame("Frame",fr_name .. "icon",fra) 
		fraicon.texture = fraicon:CreateTexture()
	end
	fraicon:SetSize(45,45)
	fraicon:SetPoint("TOPLEFT",fr_name,"TOPLEFT",3,-3)
	fraicon.texture:SetTexture(Image, "ARTWORK")
	fraicon.texture:SetAllPoints(fraicon)
	fraicon.texture:SetTexCoord(0,1,0,1)
	if Completed == true then fraicon.texture:SetVertexColor(1,1,1,1) else fraicon.texture:SetVertexColor(.55, .55, .55, 1) end
	fraicon:Show()

	local frashield = _G[fr_name .. "shield"]
	if not frashield then 
		frashield = CreateFrame("Frame",fr_name .. "shield",fra) 
		frashield.texture = frashield:CreateTexture()
	end
	frashield.texture:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Shields", "ARTWORK")
	frashield:SetSize(65,65)
	frashield:SetPoint("TOPRIGHT",fr_name,"TOPRIGHT",-1,-2)
	frashield.texture:SetAllPoints(frashield)
	if Completed == true then 
		frashield.texture:SetTexCoord(0, 0.5, 0, 0.5)
		frashield:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			if earnedByMe then GameTooltip:AddLine("Earned by: |cffffffff"..earnedBy)
			else 
				GameTooltip:AddLine("Completed by: |cffffffff"..earnedBy)
				GameTooltip:AddLine("|cffff2020Not completed by: |cffffffff"..UnitName("player"))
			end
			GameTooltip:Show()
		end)
		frashield:SetScript("OnLeave", function(self)
			GameTooltip:Hide()
		end)
	else 
		frashield.texture:SetTexCoord(0.5, 1, 0, 0.5) 
		frashield:SetScript("OnEnter", nil)
		frashield:SetScript("OnLeave", nil)

	end
	frashield:Show()

	local pts = _G[fr_name .. "shield_Points"]
	if not pts then pts = frashield:CreateFontString(fr_name .. "shield_Points","OVERLAY","GameFontNormalLarge") end
	if Completed == true then pts:SetFontObject("GameFontNormalLarge") else pts:SetFontObject("GameFontDisableLarge") end
	pts:SetPoint("CENTER",frashield,"CENTER",0,3)
	pts:SetText(Points)
	pts:Show()

	local achnm = _G[fr_name .. "AchName"]
	if not achnm then achnm = fra:CreateFontString(fr_name .. "AchName","OVERLAY","GameTooltipHeaderText") end
	if Completed == true then achnm:SetFontObject("GameFontWhite") else achnm:SetFontObject("GameFontDisable") end
	achnm:SetPoint("TOPLEFT",fra,"TOPLEFT",54,-6)
	achnm:SetText(Name)
	achnm:Show()

	local achdsc = _G[fr_name .. "AchDesc"]
	if not achdsc then achdsc = fra:CreateFontString(fr_name .. "AchDesc","OVERLAY","GameFontBlackSmall") end
	achdsc:SetFontObject("AchievementDescriptionFont")
	achdsc:SetSize(375,80)
	achdsc:SetJustifyH("LEFT"); achdsc:SetJustifyV("TOP"); 
	achdsc:CanWordWrap(1); achdsc:SetMaxLines(3);
	achdsc:SetPoint("TOPLEFT",fra,"TOPLEFT",54,-22)
	achdsc:SetText(Description)
	achdsc:Show()

end

function RaidAchFilter:ShowAch(Raid)

	local chkbox = _G["RaidAch_Frame_Option"]
	local scrollbar = _G["RaidAch_Slider"]
	RList = RAF_IDs[Raid]

	for i = 1, 68 do
		local fra = _G["ach_Frame" .. i]
		if fra then fra:Hide(); end
	end

	local maxid = 1
	local indent = 0
	franum = 1
	for i = 1, #RAF_IDs[Raid] do
		local IDNumber, Name, Points, Completed = GetAchievementInfo(RAF_IDs[Raid][i])
		if Completed == true and chkbox:GetChecked() == nil then 
			RaidAchFilter:AchAdd("ach_Frame" .. maxid, RAF_IDs[Raid][i], indent)
			maxid=maxid+1
			indent=indent-75
		elseif Completed == false then
			RaidAchFilter:AchAdd("ach_Frame" .. maxid, RAF_IDs[Raid][i], indent)
			maxid=maxid+1
			indent=indent-75
		end
	end
	maxid=(maxid*75) -- 75px per achievement box
	if maxid > 600 then maxid = maxid - 595; scrollbar:Show(); scrollbar:Enable(); 
	else maxid = 100; scrollbar:Hide(); scrollbar:Disable(); end -- hide scrollbar if not needed
	scrollbar:SetMinMaxValues(1, maxid) 
	scrollbar:SetValueStep(15.0)
	scrollbar:SetValue(0)
	RAF_RID = Raid

	local dropDown = _G["RaidAch_Dropdown"]
	UIDropDownMenu_Initialize(dropDown)
	UIDropDownMenu_SetSelectedValue(dropDown, Raid)
	UIDropDownMenu_SetText(dropDown, RAF_Names[Raid]);

end